# frozen_string_literal: true

require 'active_support'
require 'active_support/core_ext/object'
require 'sequel'
require 'roqua/csv_export/version'
require 'roqua/csv_export/models'

module Roqua
  module CsvExport
    extend ActiveSupport::Autoload

    autoload :CsvExportQuestionnaireVersion
    autoload :ExportVersions
    autoload :MetaData
    autoload :Config

    DB = Sequel.sqlite

    # Generate te export files.
    # When done, output_path will contain a directory for each export_version.
    # Each directory will contain csv file for each questionnare.
    # @see Config#initialize Config for list of attributes
    def self.generate(**attrs)
      require 'csv'
      require 'json'
      config = Config.new(**attrs)
      load_db(DB, config.sqlite_path)
      create_indexes

      FileUtils.mkdir_p config.output_path
      do_export config.export_versions, config
    end

    private

    def self.load_db(db, path)
      source = SQLite3::Database.new(path.to_s)

      db.synchronize do |memory_db|
        backup = SQLite3::Backup.new(memory_db, 'main', source, 'main')
        backup.step(-1)
        backup.finish
      end
    ensure
      source.close
    end

    def self.create_indexes
      Sequel.extension :migration
      Sequel.migration do
        change do
          alter_table(:responses) do
            add_index :questionnaire_key
          end
        end
      end
    end

    def self.questionnaire_keys
      Models::Response.distinct.select_map(:questionnaire_key)
    end

    def self.do_export(export_versions, config)
      questionnaire_keys.each do |questionnaire_key|
        responses = CsvExport::Models::Response \
                      .eager(:dossier, :team, :entered_by, :respondent, :response_values, :response_flags, :response_textvars, :team, measurement: :protocol)
                      .where(questionnaire_key: questionnaire_key)
                      .order(:id)

        CsvExportQuestionnaireVersion.with_versions_for(questionnaire_key,
                                                        config) do |versions|
          responses.paged_each do |response|
            versions.each do |version|
              version.write_response(response)
            end
          end
        end
      end
    end
  end
end
