# frozen_string_literal: true
require 'json'

module Roqua::CsvExport
  # Reads json files with export_version info and caches them.
  class ExportVersions
    def initialize(mapping_files_directory_path)
      @base_dir = mapping_files_directory_path
      @export_versions = {}
    end

    # Returns column information for specified version for each variable in each questionnaire.
    #
    # Each column_hash looks like:
    #   {"key": "tot",
    #    "sub_key": "other",
    #    "header": "all_quests_tot_o",
    #    "source_type": "score"}
    # Where header is the header within the csv file.
    #
    # @return [Hash] !{"fields" => {"oq45" => [{column_hash}, ...}}
    def [](key)
      @export_versions[key] ||= ::JSON.parse(File.read(File.join(@base_dir, "#{key}.json")))
    end

    def dig(key, *keys)
      self[key].dig(*keys)
    end

    # @return [Array<String>] all the available versions, sorted oldest first.
    def keys
      Dir["#{@base_dir}/*.json"].map {|f| File.basename(f, File.extname(f)) }.sort
    end

    # Returns mapping for the questionnaire.
    # First tries given export_version_key, but if the questonnaires wasn't defined then yet,
    # it finds the first export_version that did define it.
    # @return [Array<Hash>]
    # @see []
    def mapping_for(questionnaire_key, export_version_key)
      dig(export_version_key, 'fields', questionnaire_key) \
      || keys.lazy
             .reject { |key| key <= export_version_key }
             .map { |key| dig(key, 'fields', questionnaire_key) }
             .reject(&:nil?)
             .first
    end
  end
end
